"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Insert links to external style resources.
 */
function insertStylesheets(stylesheets, document, el) {
    stylesheets.forEach(function (href) {
        var link = document.createElement('link');
        link.type = 'text/css';
        link.rel = 'stylesheet';
        link.href = href;
        document.body.insertBefore(link, el);
    });
}
/**
 * Inserts a single stylesheet element
 */
function insertStyles(style, document, el) {
    var styleElement = document.createElement('style');
    styleElement.appendChild(document.createTextNode(style));
    document.body.insertBefore(styleElement, el);
}
function insertSingleCssFile(cssFilename, document, el) {
    return cy.readFile(cssFilename).then(function (css) {
        var style = document.createElement('style');
        style.appendChild(document.createTextNode(css));
        document.body.insertBefore(style, el);
    });
}
/**
 * Reads the given CSS file from local file system
 * and adds the loaded style text as an element.
 */
function insertLocalCssFiles(cssFilenames, document, el) {
    return Cypress.Promise.mapSeries(cssFilenames, function (cssFilename) {
        return insertSingleCssFile(cssFilename, document, el);
    });
}
/**
 * Injects custom style text or CSS file or 3rd party style resources
 * into the given document.
 */
exports.injectStylesBeforeElement = function (options, document, el) {
    // insert any custom global styles before the component
    if (typeof options.stylesheets === 'string') {
        options.stylesheets = [options.stylesheets];
    }
    if (Array.isArray(options.stylesheets)) {
        insertStylesheets(options.stylesheets, document, el);
    }
    if (options.style) {
        insertStyles(options.style, document, el);
    }
    if (Array.isArray(options.cssFiles)) {
        return insertLocalCssFiles(options.cssFiles, document, el);
    }
    if (typeof options.cssFiles === 'string') {
        return insertSingleCssFile(options.cssFiles, document, el);
    }
};
